/**
 * @jest-environment jsdom
 */

// Copyright (c) 2023 Tudor Stanciu

import {
  setItem,
  getItem,
  removeItem,
  clear,
  getKey
} from "../../storage/localStorage";

describe("localStorage", () => {
  beforeEach(() => {
    window.localStorage.clear();
  });

  test("setItem should store a string value under a given key", () => {
    setItem("myKey", "myValue");
    expect(window.localStorage.getItem("myKey")).toBe("myValue");
  });

  test("setItem should store an object value under a given key", () => {
    const obj = { foo: "bar" };
    setItem("myKey", obj);
    expect(JSON.parse(window.localStorage.getItem("myKey")!)).toEqual(obj);
  });

  test("setItem should store an array value under a given key", () => {
    const arr = [1, 2, 3];
    setItem("myKey", arr);
    expect(JSON.parse(window.localStorage.getItem("myKey")!)).toEqual(arr);
  });

  test("getItem should retrieve a stored string value under a given key", () => {
    window.localStorage.setItem("myKey", "myValue");
    expect(getItem("myKey")).toBe("myValue");
  });

  test("getItem should retrieve a stored object value under a given key", () => {
    const obj = { foo: "bar" };
    window.localStorage.setItem("myKey", JSON.stringify(obj));
    expect(getItem("myKey")).toEqual(obj);
  });

  test("getItem should retrieve a stored array value under a given key", () => {
    const arr = [1, 2, 3];
    window.localStorage.setItem("myKey", JSON.stringify(arr));
    expect(getItem("myKey")).toEqual(arr);
  });

  test("getItem should return null if key does not exist", () => {
    expect(getItem("nonexistentKey")).toBeNull();
  });

  test("removeItem should remove a value under a given key", () => {
    window.localStorage.setItem("myKey", "myValue");
    removeItem("myKey");
    expect(window.localStorage.getItem("myKey")).toBeNull();
  });

  test("clear should remove all values", () => {
    window.localStorage.setItem("myKey1", "myValue1");
    window.localStorage.setItem("myKey2", "myValue2");
    clear();
    expect(window.localStorage.getItem("myKey1")).toBeNull();
    expect(window.localStorage.getItem("myKey2")).toBeNull();
  });

  test("getKey should return the key at a given index", () => {
    window.localStorage.setItem("myKey1", "myValue1");
    window.localStorage.setItem("myKey2", "myValue2");
    expect(getKey(0)).toBe("myKey1");
    expect(getKey(1)).toBe("myKey2");
    expect(getKey(2)).toBeNull();
  });
});
